@@ -40,7 +40,6 @@
 #include <QJsonDocument>
 #include <QJsonObject>
 #include <QJsonValue>
-#include <QSaveFile>
 #include <QSet>
 #include <QTextStream>
 #include <QThread>
@@ -58,7 +57,9 @@
 #include "base/logger.h"
 #include "base/profile.h"
 #include "base/settingsstorage.h"
+#include "base/tagset.h"
 #include "base/utils/fs.h"
+#include "base/utils/io.h"
 #include "base/utils/string.h"
 
 using namespace std::chrono_literals;
@@ -73,8 +74,11 @@ const QString OPTION_RECURSIVE {QStringLiteral("recursive")};
 const QString PARAM_CATEGORY {QStringLiteral("category")};
 const QString PARAM_TAGS {QStringLiteral("tags")};
 const QString PARAM_SAVEPATH {QStringLiteral("save_path")};
+const QString PARAM_USEDOWNLOADPATH {QStringLiteral("use_download_path")};
+const QString PARAM_DOWNLOADPATH {QStringLiteral("download_path")};
 const QString PARAM_OPERATINGMODE {QStringLiteral("operating_mode")};
 const QString PARAM_STOPPED {QStringLiteral("stopped")};
+const QString PARAM_SKIPCHECKING {QStringLiteral("skip_checking")};
 const QString PARAM_CONTENTLAYOUT {QStringLiteral("content_layout")};
 const QString PARAM_AUTOTMM {QStringLiteral("use_auto_tmm")};
 const QString PARAM_UPLOADLIMIT {QStringLiteral("upload_limit")};
@@ -84,16 +88,16 @@ const QString PARAM_RATIOLIMIT {QStringLiteral("ratio_limit")};
 
 namespace
 {
-    QSet<QString> parseTagSet(const QJsonArray &jsonArr)
+    TagSet parseTagSet(const QJsonArray &jsonArr)
     {
-        QSet<QString> tags;
+        TagSet tags;
         for (const QJsonValue &jsonVal : jsonArr)
             tags.insert(jsonVal.toString());
 
         return tags;
     }
 
-    QJsonArray serializeTagSet(const QSet<QString> &tags)
+    QJsonArray serializeTagSet(const TagSet &tags)
     {
         QJsonArray arr;
         for (const QString &tag : tags)
@@ -134,8 +138,11 @@ namespace
         params.category = jsonObj.value(PARAM_CATEGORY).toString();
         params.tags = parseTagSet(jsonObj.value(PARAM_TAGS).toArray());
         params.savePath = jsonObj.value(PARAM_SAVEPATH).toString();
+        params.useDownloadPath = getOptionalBool(jsonObj, PARAM_USEDOWNLOADPATH);
+        params.downloadPath = jsonObj.value(PARAM_DOWNLOADPATH).toString();
         params.addForced = (getEnum<BitTorrent::TorrentOperatingMode>(jsonObj, PARAM_OPERATINGMODE) == BitTorrent::TorrentOperatingMode::Forced);
         params.addPaused = getOptionalBool(jsonObj, PARAM_STOPPED);
+        params.skipChecking = jsonObj.value(PARAM_SKIPCHECKING).toBool();
         params.contentLayout = getOptionalEnum<BitTorrent::TorrentContentLayout>(jsonObj, PARAM_CONTENTLAYOUT);
         params.useAutoTMM = getOptionalBool(jsonObj, PARAM_AUTOTMM);
         params.uploadLimit = jsonObj.value(PARAM_UPLOADLIMIT).toInt(-1);
@@ -152,8 +159,10 @@ namespace
             {PARAM_CATEGORY, params.category},
             {PARAM_TAGS, serializeTagSet(params.tags)},
             {PARAM_SAVEPATH, params.savePath},
+            {PARAM_DOWNLOADPATH, params.downloadPath},
             {PARAM_OPERATINGMODE, Utils::String::fromEnum(params.addForced
                 ? BitTorrent::TorrentOperatingMode::Forced : BitTorrent::TorrentOperatingMode::AutoManaged)},
+            {PARAM_SKIPCHECKING, params.skipChecking},
             {PARAM_UPLOADLIMIT, params.uploadLimit},
             {PARAM_DOWNLOADLIMIT, params.downloadLimit},
             {PARAM_SEEDINGTIMELIMIT, params.seedingTimeLimit},
@@ -166,6 +175,8 @@ namespace
             jsonObj[PARAM_CONTENTLAYOUT] = Utils::String::fromEnum(*params.contentLayout);
         if (params.useAutoTMM)
             jsonObj[PARAM_AUTOTMM] = *params.useAutoTMM;
+        if (params.useDownloadPath)
+            jsonObj[PARAM_USEDOWNLOADPATH] = *params.useDownloadPath;
 
         return jsonObj;
     }
@@ -191,7 +202,7 @@ namespace
 class TorrentFilesWatcher::Worker final : public QObject
 {
     Q_OBJECT
-    Q_DISABLE_COPY(Worker)
+    Q_DISABLE_COPY_MOVE(Worker)
 
 public:
     Worker();
@@ -206,6 +217,7 @@ signals:
 
 private:
     void onTimeout();
+    void scheduleWatchedFolderProcessing(const QString &path);
     void processWatchedFolder(const QString &path);
     void processFolder(const QString &path, const QString &watchedFolderPath, const TorrentFilesWatcher::WatchedFolderOptions &options);
     void processFailedTorrents();
@@ -368,13 +380,13 @@ void TorrentFilesWatcher::store() const
         jsonObj[watchedFolder] = serializeWatchedFolderOptions(options);
     }
 
+    const QString path = QDir(specialFolderLocation(SpecialFolder::Config)).absoluteFilePath(CONF_FILE_NAME);
     const QByteArray data = QJsonDocument(jsonObj).toJson();
-
-    QSaveFile confFile {QDir(specialFolderLocation(SpecialFolder::Config)).absoluteFilePath(CONF_FILE_NAME)};
-    if (!confFile.open(QIODevice::WriteOnly) || (confFile.write(data) != data.size()) || !confFile.commit())
+    const nonstd::expected<void, QString> result = Utils::IO::saveToFile(path, data);
+    if (!result)
     {
         LogMsg(tr("Couldn't store Watched Folders configuration to %1. Error: %2")
-            .arg(confFile.fileName(), confFile.errorString()), Log::WARNING);
+            .arg(path, result.error()), Log::WARNING);
     }
 }
 
@@ -435,7 +447,7 @@ TorrentFilesWatcher::Worker::Worker()
     , m_watchTimer {new QTimer(this)}
     , m_retryTorrentTimer {new QTimer(this)}
 {
-    connect(m_watcher, &QFileSystemWatcher::directoryChanged, this, &Worker::processWatchedFolder);
+    connect(m_watcher, &QFileSystemWatcher::directoryChanged, this, &Worker::scheduleWatchedFolderProcessing);
     connect(m_watchTimer, &QTimer::timeout, this, &Worker::onTimeout);
 
     connect(m_retryTorrentTimer, &QTimer::timeout, this, &Worker::processFailedTorrents);
@@ -469,6 +481,14 @@ void TorrentFilesWatcher::Worker::removeWatchedFolder(const QString &path)
         m_retryTorrentTimer->stop();
 }
 
+void TorrentFilesWatcher::Worker::scheduleWatchedFolderProcessing(const QString &path)
+{
+    QTimer::singleShot(2000, this, [this, path]()
+    {
+        processWatchedFolder(path);
+    });
+}
+
 void TorrentFilesWatcher::Worker::processWatchedFolder(const QString &path)
 {
     const TorrentFilesWatcher::WatchedFolderOptions options = m_watchedFolders.value(path);
@@ -491,7 +511,15 @@ void TorrentFilesWatcher::Worker::processFolder(const QString &path, const QStri
         if (path != watchedFolderPath)
         {
             const QString subdirPath = watchedDir.relativeFilePath(path);
-            addTorrentParams.savePath = QDir::cleanPath(QDir(addTorrentParams.savePath).filePath(subdirPath));
+            if (addTorrentParams.useAutoTMM)
+            {
+                addTorrentParams.category = addTorrentParams.category.isEmpty()
+                        ? subdirPath : (addTorrentParams.category + QLatin1Char('/') + subdirPath);
+            }
+            else
+            {
+                addTorrentParams.savePath = QDir::cleanPath(QDir(addTorrentParams.savePath).filePath(subdirPath));
+            }
         }
 
         if (filePath.endsWith(QLatin1String(".magnet"), Qt::CaseInsensitive))
@@ -513,10 +541,10 @@ void TorrentFilesWatcher::Worker::processFolder(const QString &path, const QStri
         }
         else
         {
-            const auto torrentInfo = BitTorrent::TorrentInfo::loadFromFile(filePath);
-            if (torrentInfo.isValid())
+            const nonstd::expected<BitTorrent::TorrentInfo, QString> result = BitTorrent::TorrentInfo::loadFromFile(filePath);
+            if (result)
             {
-                emit torrentFound(torrentInfo, addTorrentParams);
+                emit torrentFound(result.value(), addTorrentParams);
                 Utils::Fs::forceRemove(filePath);
             }
             else
@@ -554,18 +582,26 @@ void TorrentFilesWatcher::Worker::processFailedTorrents()
             if (!QFile::exists(torrentPath))
                 return true;
 
-            const auto torrentInfo = BitTorrent::TorrentInfo::loadFromFile(torrentPath);
-            if (torrentInfo.isValid())
+            const nonstd::expected<BitTorrent::TorrentInfo, QString> result = BitTorrent::TorrentInfo::loadFromFile(torrentPath);
+            if (result)
             {
                 BitTorrent::AddTorrentParams addTorrentParams = options.addTorrentParams;
                 const QString exactDirPath = QFileInfo(torrentPath).canonicalPath();
                 if (exactDirPath != dir.path())
                 {
                     const QString subdirPath = dir.relativeFilePath(exactDirPath);
-                    addTorrentParams.savePath = QDir(addTorrentParams.savePath).filePath(subdirPath);
+                    if (addTorrentParams.useAutoTMM)
+                    {
+                        addTorrentParams.category = addTorrentParams.category.isEmpty()
+                                ? subdirPath : (addTorrentParams.category + QLatin1Char('/') + subdirPath);
+                    }
+                    else
+                    {
+                        addTorrentParams.savePath = QDir(addTorrentParams.savePath).filePath(subdirPath);
+                    }
                 }
 
-                emit torrentFound(torrentInfo, addTorrentParams);
+                emit torrentFound(result.value(), addTorrentParams);
                 Utils::Fs::forceRemove(torrentPath);
 
                 return true;
@@ -611,7 +647,7 @@ void TorrentFilesWatcher::Worker::addWatchedFolder(const QString &path, const To
     else
     {
         m_watcher->addPath(path);
-        QTimer::singleShot(2000, this, [this, path]() { processWatchedFolder(path); });
+        scheduleWatchedFolderProcessing(path);
     }
 
     m_watchedFolders[path] = options;
@@ -643,7 +679,7 @@ void TorrentFilesWatcher::Worker::updateWatchedFolder(const QString &path, const
                 m_watchTimer->stop();
 
             m_watcher->addPath(path);
-            QTimer::singleShot(2000, this, [this, path]() { processWatchedFolder(path); });
+            scheduleWatchedFolderProcessing(path);
         }
     }
 
