@@ -36,7 +36,6 @@
 #include <QHeaderView>
 #include <QMenu>
 #include <QMessageBox>
-#include <QRegExp>
 #include <QRegularExpression>
 #include <QSet>
 #include <QShortcut>
@@ -45,7 +44,6 @@
 #include <QWheelEvent>
 
 #include "base/bittorrent/common.h"
-#include "base/bittorrent/infohash.h"
 #include "base/bittorrent/session.h"
 #include "base/bittorrent/torrent.h"
 #include "base/bittorrent/trackerentry.h"
@@ -53,6 +51,7 @@
 #include "base/logger.h"
 #include "base/preferences.h"
 #include "base/torrentfilter.h"
+#include "base/utils/compare.h"
 #include "base/utils/fs.h"
 #include "base/utils/misc.h"
 #include "base/utils/string.h"
@@ -92,9 +91,9 @@ namespace
         if (!torrent->hasMetadata())
             return false;
 
-        for (int i = 0; i < torrent->filesCount(); ++i)
+        for (const QString &filePath : asConst(torrent->filePaths()))
         {
-            QString fileName = torrent->fileName(i);
+            QString fileName = Utils::Fs::fileName(filePath);
             if (fileName.endsWith(QB_EXT))
                 fileName.chop(QB_EXT.length());
             if (Utils::Misc::isPreviewable(fileName))
@@ -107,12 +106,12 @@ namespace
     void openDestinationFolder(const BitTorrent::Torrent *const torrent)
     {
 #ifdef Q_OS_MACOS
-        MacUtils::openFiles({torrent->contentPath(true)});
+        MacUtils::openFiles({torrent->contentPath()});
 #else
         if (torrent->filesCount() == 1)
-            Utils::Gui::openFolderSelect(torrent->contentPath(true));
+            Utils::Gui::openFolderSelect(torrent->contentPath());
         else
-            Utils::Gui::openPath(torrent->contentPath(true));
+            Utils::Gui::openPath(torrent->contentPath());
 #endif
     }
 
@@ -158,6 +157,7 @@ TransferListWidget::TransferListWidget(QWidget *parent, MainWindow *mainWindow)
     setAttribute(Qt::WA_MacShowFocusRect, false);
 #endif
     header()->setStretchLastSection(false);
+    header()->setTextElideMode(Qt::ElideRight);
 
     // Default hidden columns
     if (!columnLoaded)
@@ -303,6 +303,9 @@ void TransferListWidget::torrentDoubleClicked()
     case OPEN_DEST:
         openDestinationFolder(torrent);
         break;
+    case SHOW_OPTIONS:
+        setTorrentOptions();
+        break;
     }
 }
 
@@ -328,21 +331,6 @@ QVector<BitTorrent::Torrent *> TransferListWidget::getVisibleTorrents() const
     return torrents;
 }
 
-void TransferListWidget::setSelectedTorrentsLocation()
-{
-    const QVector<BitTorrent::Torrent *> torrents = getSelectedTorrents();
-    if (torrents.isEmpty()) return;
-
-    const QString oldLocation = torrents[0]->savePath();
-    const QString newLocation = QFileDialog::getExistingDirectory(this, tr("Choose save path"), oldLocation,
-                                            QFileDialog::DontConfirmOverwrite | QFileDialog::ShowDirsOnly | QFileDialog::HideNameFilterDetails);
-    if (newLocation.isEmpty() || !QDir(newLocation).exists()) return;
-
-    // Actually move storage
-    for (BitTorrent::Torrent *const torrent : torrents)
-        torrent->move(Utils::Fs::expandPathAbs(newLocation));
-}
-
 void TransferListWidget::pauseAllTorrents()
 {
     for (BitTorrent::Torrent *const torrent : asConst(BitTorrent::Session::instance()->torrents()))
@@ -487,7 +475,33 @@ void TransferListWidget::copySelectedNames() const
     qApp->clipboard()->setText(torrentNames.join('\n'));
 }
 
-void TransferListWidget::copySelectedHashes() const
+void TransferListWidget::copySelectedInfohashes(const CopyInfohashPolicy policy) const
+{
+    const auto selectedTorrents = getSelectedTorrents();
+    QStringList infoHashes;
+    infoHashes.reserve(selectedTorrents.size());
+    switch (policy)
+    {
+    case CopyInfohashPolicy::Version1:
+        for (const BitTorrent::Torrent *torrent : selectedTorrents)
+        {
+            if (const auto infoHash = torrent->infoHash().v1(); infoHash.isValid())
+                infoHashes << infoHash.toString();
+        }
+        break;
+    case CopyInfohashPolicy::Version2:
+        for (const BitTorrent::Torrent *torrent : selectedTorrents)
+        {
+            if (const auto infoHash = torrent->infoHash().v2(); infoHash.isValid())
+                infoHashes << infoHash.toString();
+        }
+        break;
+    }
+
+    qApp->clipboard()->setText(infoHashes.join('\n'));
+}
+
+void TransferListWidget::copySelectedIDs() const
 {
     QStringList torrentIDs;
     for (BitTorrent::Torrent *const torrent : asConst(getSelectedTorrents()))
@@ -511,22 +525,22 @@ void TransferListWidget::openSelectedTorrentsFolder() const
     // folders prehilighted for opening, so we use a custom method.
     for (BitTorrent::Torrent *const torrent : asConst(getSelectedTorrents()))
     {
-        QString path = torrent->contentPath(true);
-        pathsList.insert(path);
+        const QString contentPath = QDir(torrent->actualStorageLocation()).absoluteFilePath(torrent->contentPath());
+        pathsList.insert(contentPath);
     }
     MacUtils::openFiles(pathsList);
 #else
     for (BitTorrent::Torrent *const torrent : asConst(getSelectedTorrents()))
     {
-        QString path = torrent->contentPath(true);
-        if (!pathsList.contains(path))
+        const QString contentPath = torrent->contentPath();
+        if (!pathsList.contains(contentPath))
         {
             if (torrent->filesCount() == 1)
-                Utils::Gui::openFolderSelect(path);
+                Utils::Gui::openFolderSelect(contentPath);
             else
-                Utils::Gui::openPath(path);
+                Utils::Gui::openPath(contentPath);
         }
-        pathsList.insert(path);
+        pathsList.insert(contentPath);
     }
 #endif // Q_OS_MACOS
 }
@@ -633,18 +647,6 @@ void TransferListWidget::setSelectedTorrentsSuperSeeding(const bool enabled) con
     }
 }
 
-void TransferListWidget::setSelectedTorrentsSequentialDownload(const bool enabled) const
-{
-    for (BitTorrent::Torrent *const torrent : asConst(getSelectedTorrents()))
-        torrent->setSequentialDownload(enabled);
-}
-
-void TransferListWidget::setSelectedFirstLastPiecePrio(const bool enabled) const
-{
-    for (BitTorrent::Torrent *const torrent : asConst(getSelectedTorrents()))
-        torrent->setFirstLastPiecePriority(enabled);
-}
-
 void TransferListWidget::setSelectedAutoTMMEnabled(const bool enabled) const
 {
     for (BitTorrent::Torrent *const torrent : asConst(getSelectedTorrents()))
@@ -721,7 +723,7 @@ QStringList TransferListWidget::askTagsForSelection(const QString &dialogTitle)
             this, dialogTitle, tr("Comma-separated tags:"), QLineEdit::Normal, "", &ok).trimmed();
         if (!ok || tagsInput.isEmpty())
             return {};
-        tags = tagsInput.split(',', QString::SkipEmptyParts);
+        tags = tagsInput.split(',', Qt::SkipEmptyParts);
         for (QString &tag : tags)
         {
             tag = tag.trimmed();
@@ -819,29 +821,24 @@ void TransferListWidget::displayListMenu(const QPoint &)
     connect(actionTopQueuePos, &QAction::triggered, this, &TransferListWidget::topQueuePosSelectedTorrents);
     auto *actionBottomQueuePos = new QAction(UIThemeManager::instance()->getIcon("go-bottom"), tr("Move to bottom", "i.e. Move to bottom of the queue"), listMenu);
     connect(actionBottomQueuePos, &QAction::triggered, this, &TransferListWidget::bottomQueuePosSelectedTorrents);
-    auto *actionSetTorrentPath = new QAction(UIThemeManager::instance()->getIcon("inode-directory"), tr("Set location..."), listMenu);
-    connect(actionSetTorrentPath, &QAction::triggered, this, &TransferListWidget::setSelectedTorrentsLocation);
     auto *actionForceRecheck = new QAction(UIThemeManager::instance()->getIcon("document-edit-verify"), tr("Force recheck"), listMenu);
     connect(actionForceRecheck, &QAction::triggered, this, &TransferListWidget::recheckSelectedTorrents);
     auto *actionForceReannounce = new QAction(UIThemeManager::instance()->getIcon("document-edit-verify"), tr("Force reannounce"), listMenu);
     connect(actionForceReannounce, &QAction::triggered, this, &TransferListWidget::reannounceSelectedTorrents);
     auto *actionCopyMagnetLink = new QAction(UIThemeManager::instance()->getIcon("kt-magnet"), tr("Magnet link"), listMenu);
     connect(actionCopyMagnetLink, &QAction::triggered, this, &TransferListWidget::copySelectedMagnetURIs);
+    auto *actionCopyID = new QAction(UIThemeManager::instance()->getIcon("edit-copy"), tr("Torrent ID"), listMenu);
+    connect(actionCopyID, &QAction::triggered, this, &TransferListWidget::copySelectedIDs);
     auto *actionCopyName = new QAction(UIThemeManager::instance()->getIcon("edit-copy"), tr("Name"), listMenu);
     connect(actionCopyName, &QAction::triggered, this, &TransferListWidget::copySelectedNames);
-    auto *actionCopyHash = new QAction(UIThemeManager::instance()->getIcon("edit-copy"), tr("Hash"), listMenu);
-    connect(actionCopyHash, &QAction::triggered, this, &TransferListWidget::copySelectedHashes);
+    auto *actionCopyHash1 = new QAction(UIThemeManager::instance()->getIcon("edit-copy"), tr("Info hash v1"), listMenu);
+    connect(actionCopyHash1, &QAction::triggered, this, [this]() { copySelectedInfohashes(CopyInfohashPolicy::Version1); });
+    auto *actionCopyHash2 = new QAction(UIThemeManager::instance()->getIcon("edit-copy"), tr("Info hash v2"), listMenu);
+    connect(actionCopyHash2, &QAction::triggered, this, [this]() { copySelectedInfohashes(CopyInfohashPolicy::Version2); });
     auto *actionSuperSeedingMode = new TriStateAction(tr("Super seeding mode"), listMenu);
     connect(actionSuperSeedingMode, &QAction::triggered, this, &TransferListWidget::setSelectedTorrentsSuperSeeding);
     auto *actionRename = new QAction(UIThemeManager::instance()->getIcon("edit-rename"), tr("Rename..."), listMenu);
     connect(actionRename, &QAction::triggered, this, &TransferListWidget::renameSelectedTorrent);
-    auto *actionSequentialDownload = new TriStateAction(tr("Download in sequential order"), listMenu);
-    connect(actionSequentialDownload, &QAction::triggered, this, &TransferListWidget::setSelectedTorrentsSequentialDownload);
-    auto *actionFirstLastPiecePrio = new TriStateAction(tr("Download first and last pieces first"), listMenu);
-    connect(actionFirstLastPiecePrio, &QAction::triggered, this, &TransferListWidget::setSelectedFirstLastPiecePrio);
-    auto *actionAutoTMM = new TriStateAction(tr("Automatic Torrent Management"), listMenu);
-    actionAutoTMM->setToolTip(tr("Automatic mode means that various torrent properties(eg save path) will be decided by the associated category"));
-    connect(actionAutoTMM, &QAction::triggered, this, &TransferListWidget::setSelectedAutoTMMEnabled);
     auto *actionEditTracker = new QAction(UIThemeManager::instance()->getIcon("edit-rename"), tr("Edit trackers..."), listMenu);
     connect(actionEditTracker, &QAction::triggered, this, &TransferListWidget::editTorrentTrackers);
     // End of actions
@@ -854,12 +851,11 @@ void TransferListWidget::displayListMenu(const QPoint &)
     bool sequentialDownloadMode = false, prioritizeFirstLast = false;
     bool oneHasMetadata = false, oneNotSeed = false;
     bool allSameCategory = true;
-    bool allSameAutoTMM = true;
-    bool firstAutoTMM = false;
     QString firstCategory;
     bool first = true;
-    QSet<QString> tagsInAny;
-    QSet<QString> tagsInAll;
+    TagSet tagsInAny;
+    TagSet tagsInAll;
+    bool hasInfohashV1 = false, hasInfohashV2 = false;
 
     for (const QModelIndex &index : selectedIndexes)
     {
@@ -873,21 +869,18 @@ void TransferListWidget::displayListMenu(const QPoint &)
         if (firstCategory != torrent->category())
             allSameCategory = false;
 
-        tagsInAny.unite(torrent->tags());
+        const TagSet torrentTags = torrent->tags();
+        tagsInAny.unite(torrentTags);
 
         if (first)
         {
-            firstAutoTMM = torrent->isAutoTMMEnabled();
-            tagsInAll = torrent->tags();
+            tagsInAll = torrentTags;
         }
         else
         {
-            tagsInAll.intersect(torrent->tags());
+            tagsInAll.intersect(torrentTags);
         }
 
-        if (firstAutoTMM != torrent->isAutoTMMEnabled())
-            allSameAutoTMM = false;
-
         if (torrent->hasMetadata())
             oneHasMetadata = true;
         if (!torrent->isSeed())
@@ -938,12 +931,18 @@ void TransferListWidget::displayListMenu(const QPoint &)
         if (torrent->hasMetadata())
             needsPreview = true;
 
+        if (!hasInfohashV1 && torrent->infoHash().v1().isValid())
+            hasInfohashV1 = true;
+        if (!hasInfohashV2 && torrent->infoHash().v2().isValid())
+            hasInfohashV2 = true;
+
         first = false;
 
         if (oneHasMetadata && oneNotSeed && !allSameSequentialDownloadMode
             && !allSamePrioFirstlast && !allSameSuperSeeding && !allSameCategory
-            && needsStart && needsForce && needsPause && needsPreview && !allSameAutoTMM)
-            {
+            && needsStart && needsForce && needsPause && needsPreview
+            && hasInfohashV1 && hasInfohashV2)
+        {
             break;
         }
     }
@@ -957,39 +956,13 @@ void TransferListWidget::displayListMenu(const QPoint &)
     listMenu->addSeparator();
     listMenu->addAction(actionDelete);
     listMenu->addSeparator();
-    listMenu->addAction(actionSetTorrentPath);
     if (selectedIndexes.size() == 1)
         listMenu->addAction(actionRename);
     listMenu->addAction(actionEditTracker);
 
-    // Category Menu
-    QStringList categories = BitTorrent::Session::instance()->categories().keys();
-    std::sort(categories.begin(), categories.end(), Utils::String::naturalLessThan<Qt::CaseInsensitive>);
-
-    QMenu *categoryMenu = listMenu->addMenu(UIThemeManager::instance()->getIcon("view-categories"), tr("Category"));
-
-    categoryMenu->addAction(UIThemeManager::instance()->getIcon("list-add"), tr("New...", "New category...")
-        , this, &TransferListWidget::askNewCategoryForSelection);
-    categoryMenu->addAction(UIThemeManager::instance()->getIcon("edit-clear"), tr("Reset", "Reset category")
-        , this, [this]() { setSelectionCategory(""); });
-    categoryMenu->addSeparator();
-
-    for (const QString &category : asConst(categories))
-    {
-        const QString escapedCategory = QString(category).replace('&', "&&");  // avoid '&' becomes accelerator key
-        QAction *cat = categoryMenu->addAction(UIThemeManager::instance()->getIcon("inode-directory"), escapedCategory
-            , this, [this, category]() { setSelectionCategory(category); });
-
-        if (allSameCategory && (category == firstCategory))
-        {
-            cat->setCheckable(true);
-            cat->setChecked(true);
-        }
-    }
-
     // Tag Menu
     QStringList tags(BitTorrent::Session::instance()->tags().values());
-    std::sort(tags.begin(), tags.end(), Utils::String::naturalLessThan<Qt::CaseInsensitive>);
+    std::sort(tags.begin(), tags.end(), Utils::Compare::NaturalLessThan<Qt::CaseInsensitive>());
 
     QMenu *tagsMenu = listMenu->addMenu(UIThemeManager::instance()->getIcon("view-categories"), tr("Tags"));
 
@@ -1011,8 +984,7 @@ void TransferListWidget::displayListMenu(const QPoint &)
         action->setCloseOnInteraction(false);
 
         const Qt::CheckState initialState = tagsInAll.contains(tag) ? Qt::Checked
-                                            : tagsInAny.contains(tag) ? Qt::PartiallyChecked
-                                            : Qt::Unchecked;
+                                            : tagsInAny.contains(tag) ? Qt::PartiallyChecked : Qt::Unchecked;
         action->setCheckState(initialState);
 
         connect(action, &QAction::toggled, this, [this, tag](const bool checked)
@@ -1026,11 +998,6 @@ void TransferListWidget::displayListMenu(const QPoint &)
         tagsMenu->addAction(action);
     }
 
-    actionAutoTMM->setCheckState(allSameAutoTMM
-        ? (firstAutoTMM ? Qt::Checked : Qt::Unchecked)
-        : Qt::PartiallyChecked);
-    listMenu->addAction(actionAutoTMM);
-
     listMenu->addSeparator();
     listMenu->addAction(actionTorrentOptions);
     if (!oneNotSeed && oneHasMetadata)
@@ -1048,19 +1015,7 @@ void TransferListWidget::displayListMenu(const QPoint &)
         addedPreviewAction = true;
     }
     if (oneNotSeed)
-    {
-        actionSequentialDownload->setCheckState(allSameSequentialDownloadMode
-            ? (sequentialDownloadMode ? Qt::Checked : Qt::Unchecked)
-            : Qt::PartiallyChecked);
-        listMenu->addAction(actionSequentialDownload);
-
-        actionFirstLastPiecePrio->setCheckState(allSamePrioFirstlast
-            ? (prioritizeFirstLast ? Qt::Checked : Qt::Unchecked)
-            : Qt::PartiallyChecked);
-        listMenu->addAction(actionFirstLastPiecePrio);
-
         addedPreviewAction = true;
-    }
 
     if (addedPreviewAction)
         listMenu->addSeparator();
@@ -1084,8 +1039,12 @@ void TransferListWidget::displayListMenu(const QPoint &)
     QMenu *copySubMenu = listMenu->addMenu(
         UIThemeManager::instance()->getIcon("edit-copy"), tr("Copy"));
     copySubMenu->addAction(actionCopyName);
-    copySubMenu->addAction(actionCopyHash);
+    copySubMenu->addAction(actionCopyHash1);
+    actionCopyHash1->setEnabled(hasInfohashV1);
+    copySubMenu->addAction(actionCopyHash2);
+    actionCopyHash2->setEnabled(hasInfohashV2);
     copySubMenu->addAction(actionCopyMagnetLink);
+    copySubMenu->addAction(actionCopyID);
 
     listMenu->popup(QCursor::pos());
 }
@@ -1131,9 +1090,9 @@ void TransferListWidget::applyTrackerFilter(const QSet<BitTorrent::TorrentID> &t
 
 void TransferListWidget::applyNameFilter(const QString &name)
 {
-    const QRegExp::PatternSyntax patternSyntax = Preferences::instance()->getRegexAsFilteringPatternForTransferList()
-                ? QRegExp::RegExp : QRegExp::WildcardUnix;
-    m_sortFilterModel->setFilterRegExp(QRegExp(name, Qt::CaseInsensitive, patternSyntax));
+    const QString pattern = (Preferences::instance()->getRegexAsFilteringPatternForTransferList()
+                ? name : Utils::String::wildcardToRegexPattern(name));
+    m_sortFilterModel->setFilterRegularExpression(QRegularExpression(pattern, QRegularExpression::CaseInsensitiveOption));
 }
 
 void TransferListWidget::applyStatusFilter(int f)
@@ -1163,15 +1122,9 @@ void TransferListWidget::wheelEvent(QWheelEvent *event)
     {
         // Shift + scroll = horizontal scroll
         event->accept();
-
-#if (QT_VERSION >= QT_VERSION_CHECK(5, 14, 0))
         QWheelEvent scrollHEvent(event->position(), event->globalPosition()
             , event->pixelDelta(), event->angleDelta().transposed(), event->buttons()
             , event->modifiers(), event->phase(), event->inverted(), event->source());
-#else
-        QWheelEvent scrollHEvent(event->pos(), event->globalPos()
-            , event->delta(), event->buttons(), event->modifiers(), Qt::Horizontal);
-#endif
         QTreeView::wheelEvent(&scrollHEvent);
         return;
     }
